-- 使用 entity mesh 时，不需要 myShader
-- 使用 pure mesh 时，需要 myShader 计算光照
-- 简单定义形式必须严格使用 shader([[...]], [[...]])，不能使用 shader{vertex=[[...]], fragment=[[...]]}}

---[=[

VisualWorkGroup = shader.compute(
[[
#version 430
#include <codea/common.glsl>

uniform uint instanceCount;


// 
struct Instance
{
vec4 position;
};

/*
layout(std430, binding = BUFFER0 ) buffer  Pos
{
    Instance instances[ ]; 
}; 
*/
// 定义 RW 型的 buffer，变量名为 instances，单元类型为结构体 Instance，使用 BUFFER0 缓存位置 
BUFFER_RW(instances, Instance, BUFFER0);

//输入布局限定符声明一个 16x8x1 的本地工作组----gl_WorkGroupSize
//layout( local_size_x = 1, local_size_y = 1, local_size_z = 1 ) in;
NUM_THREADS(2, 2, 2)

void main()     
{

// 当前工作项在本地工作组中的位置偏移向量
vec3 offsetItem = gl_LocalInvocationID.xyz;

// 当前工作项所在的当前工作组在全局工作组中的位置偏移向量
//vec3 offsetWorkGroup = gl_WorkGroupID.xyz;

// 当前工作项在全局工作组中的位置偏移向量
vec3 gId = gl_GlobalInvocationID;
vec3 lSize = gl_WorkGroupSize;
vec3 offsetWorkGroup = vec3(gId.x * lSize.x, gId.y * lSize.y, gId.z * lSize.z);

uint factor = 5;

// 利用内置变量实例ID: gl_GlobalInvocationID 作为 buffer 索引
// Get the current instance to work on
uint i = gl_LocalInvocationIndex;

// Prevent going over the end of the buffer
if (i < instanceCount)
{
    // 计算得到当前工作项在本地工作组中的的偏移量，可以绘制出一个本地工作组
    //instances[i].position = vec4(offsetItem  * factor, 1.0);
    //instances[i].position = vec4(offsetWorkGroup  * factor, 1.0);
    //instances[i].position = vec4((offsetItem + offsetWorkGroup) * factor, 1.0);
    instances[i].position = vec4(offsetItem + offsetWorkGroup, 1.0);
}

}                    
]]
)

