-- 使用 instance entity mesh，模拟 GPU compute shadder 的工作组如何分配工作项
---- 可视化本地工作组，由多个小立方体组成的形体，形体模样取决于compute shader 中 NUM_THREADS(x,y,z) 的取值
---- 每个实例代表一个工作项(线程)，把当前本地工作组索引值作为每个实例的位置偏移量
---- 可视化全局工作组，由多个本地工作组组成的形体，形体模样取决于 dispatch 中的 vec3 的值

-- bug：透视错误，后面的没有被前面的遮挡

---[=[

instancing = shader (
[[
#version 430
#include <codea/common.glsl>

layout (location = POSITION) in vec3 a_position;
layout (location = TEXCOORD0) in vec2 a_texcoord0;

out vec2 uv;
out vec3 col;

// 利用 instances buffer 接收 compute shader 的计算结果
struct Instance
{
    vec4 position;
};
BUFFER_RW(instances, Instance, BUFFER0);

void main()
{
    vec3 posOffset = instances[gl_InstanceID].position.xyz;
    vec3 pos = a_position + posOffset;
    //pos = pos + posOffset;
    col = normalize(pos);
    gl_Position = u_modelViewProj * vec4(pos, 1.0);
    //gl_Position = u_modelViewProj * vec4(posOffset, 1.0);
    //gl_Position = u_modelViewProj * vec4(a_position + vec3(instances[gl_InstanceID].position.xyz), 1.0);
    //gl_Position = u_modelViewProj * vec4(a_position, 1.0);
    uv = a_texcoord0;
}
]],

[[
#version 430
#include <codea/common.glsl>

in vec2 uv;
in vec3 col;
out vec4 FragColor;

uniform sampler2D sTexture;//纹理内容数据
uniform float utexCoorOffset;//纹理坐标偏移
uniform vec4 utintColor;

void main()
{
//计算纹理坐标的偏移      
vec2 texCoorTemp=fract(uv + vec2(utexCoorOffset, 0.0));
//给此片元从纹理中采样出颜色值      
vec4 finalColor=texture(sTexture, texCoorTemp); 
//vec4 finalColor = texture(sTexture, uv); 
FragColor = finalColor*vec4(col.xyz, 1.0);
//FragColor = finalColor*vec4(col.xyz, 1.0)*utintColor;
}
]])

function setup()
    -- 使用基本场景的默认镜头
    scn = scene.default3d()
    scn.sky = nil
    -- scn.camera.farPlane =5000
    
    cam = scn.camera:get(camera)
    cam.farPlane = 50000
    print("cam: ", cam.farPlane)
    
    --[[ create a sun entity and add a directional light to it
    sunEntity = scn:entity("sun")
    sun = sunEntity:add(light.directional(vec3(-1,-1,-1)))
    sun.color = color(233, 150, 80)
    -- sun.clearColor = color(233, 150, 80)
    sun.intensity = 0.05   
    -- sun.direction =vec3(-100,-100,-100)
    sun.castShadows = false
    --]]
    
    
    local rig = scn.camera:add(camera.rigs.orbit)
    rig.distance = 100
    rig.angles.x = 45    
    rig.angles.y = -75
    
    -- 地面
    ground = scn:entity("ground")
    ground:add(mesh.box(50, 0.1, 50))
    -- ground:add(physics3d.body, STATIC):box(50,1,50)
    -- (this will work in the next build)
    ground.material.map = asset.documents.Dropbox.n0
    ground.material.normalMap = asset.documents.Dropbox.n2
    -- ground.material.occlusionMap= asset.documents.Dropbox.dd
    ground.y = -1.5
    
    -- 新建 material
    checkers = material.unlit()
    checkers.map = image.read(asset.documents.checker_png)
    checkers.color = color(79, 226, 18)
    checkers.scaleOffset = vec4(0.125, 0.125, 0.0, 0.0)
    checkers.roughness = 0.5
    -- checkers.shadowCascade.size = 1
    
    
    -- Create a buffer to match the shader buffer (syntax is a bit ugly right now)
    local layout = shader.bufferLayout
    {
        {shader.position, 4, shader.float} -- vec4
    }
    -- Create buffer to hold instance data
    instancesBuffer = shader.buffer(layout)
    
    initBuffer()
    
    VisualWorkGroup:setBuffer("instancesBuffer", instancesBuffer, shader.readwrite)
    VisualWorkGroup.instanceCount = instancesBuffer.size
    
    instancing:setBuffer("instancesBuffer", instancesBuffer, shader.readwrite)
    
    print("buffer size: ", instancesBuffer.size)
    print("compute group size: ", VisualWorkGroup:workgroupSize())
    
    
    texCoorOffset=0.0
    instancing.sTexture = image.read(asset.builtin.Blocks.Brick_Red)
    instancing.utexCoorOffset =1.5
    instancing.utintColor = color(99, 255, 0)
    
    --[[ create a sphere mesh
    msh = mesh.sphere(1.3,32,16,0,360,0,180)
    -- msh.material = checkers
    msh.shader = instancing    
    
    -- 新建entity mesh
    eMesh = scn:entity("eMesh")
    eMesh.mesh = mesh.sphere(1.3,32,16,0,360,0,180)
    -- eMesh.mesh = mesh.read(asset.builtin.Blocky_Characters.Adventurer)
    eMesh.mesh.shader = instancing
    --]]
    
    -- 新建一个使用 updatePositions 作为 compute shader 的 mesh
    eMesh1 = scn:entity("eMesh")
    -- eMesh1.mesh = mesh.sphere(6.3,32,16,0,360,0,180)
    eMesh1.mesh = mesh.box(vec3(1,1,1), vec3(14,4,4))
    -- eMesh1.mesh = mesh.read(asset.builtin.Blocky_Characters.Adventurer)
    eMesh1.mesh.shader = instancing
    eMesh1.active = false
    
    parameter.action("clear buffer", function() instancesBuffer:clear() end)
    parameter.action("init buffer", function() initBuffer() end)
    
    scene.main = scn
end

function initBuffer()
    -- Create initial instances by appending data to buffer
    for k = 1, 64 do
        -- instancesBuffer:append(shader.position, vec4(math.random(0, WIDTH/2),math.random(0, HEIGHT/2),1,1))
        instancesBuffer:append(shader.position, vec4(0.0, 0.0, 0.0, 0.0)) 
    end
end

function draw()
    background(0)
    
    ---[[ 根据工作组总数(所有工作项，也就是线程的数目) workgroupSize，来分配任务到 GPU 上
    local wx,wy,wz = VisualWorkGroup:workgroupSize()
    -- VisualWorkGroup:dispatch(instancesBuffer.size/(wx*wy*wz), 1, 1)
    VisualWorkGroup:dispatch(2,2,2)
    --]]    
    
    texCoorOffset = (texCoorOffset+0.01)%10.0;
    instancing.utexCoorOffset = texCoorOffset
    
    matrix.push()
    camera.perspective(45, 0.1, 1000)
    -- matrix.translate(WIDTH/2, HEIGHT/2, 100)
    -- eMesh.mesh:draw(instancesBuffer.size)
    eMesh1.mesh:draw(instancesBuffer.size)
    -- msh:draw(instancesBuffer.size)
    matrix.pop()
end

function touched(touch)
    if touch.state == BEGAN then
        local wx,wy,wz = VisualWorkGroup:workgroupSize()
        print("workgroupSize: ", wx,wy,wz)
        -- instancesBuffer:clear()
    end
end

--]=]
